#!/opt/alt/python37/bin/python3 -bb
#coding:utf-8

# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENSE.TXT
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import os
import re
import sys
import json
import shutil
import argparse
import subprocess
import configparser
import cldetectlib as detect
from clcommon.utils import get_rhn_systemid_value
from clcommon.lib.cledition import is_cl_solo_edition
from clcommon.ui_config import UIConfig

SOURCE_PATH = "/usr/share/lvemanager-xray/"
LVEMANAGER_SOURCE_PATH = "/usr/share/l.v.e-manager/"
XRAY_MANAGER_UTILITY = "/usr/sbin/cloudlinux-xray-manager"
DYNAMICUI_SYNC_CONFIG_COMMAND = "/usr/share/l.v.e-manager/utils/dynamicui.py --sync-conf=all --silent"
CLOUDLINUX_CONFIG_COMMAND = "/usr/sbin/cloudlinux-config set --data '{\"options\": {\"uiSettings\": {\"hideXrayApp\": true}}}' --json"
PANEL_INTEGRATION_CONFIG = '/opt/cpvendor/etc/integration.ini'


class Base:
    def __init__(self):
        pass

    def exec_command(self, command, env=None):
        """
        This function will run passed command
        in command line and returns result
        :param command:
        :param env:
        :return:
        """
        result = []
        try:
            p = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, env=env, text=True)
            while 1:
                output = p.stdout.readline()
                if not output:
                    break
                if output.strip() != "":
                    result.append(output.strip())
        except Exception as e:
            print ("Call process error: " + str(e))
        return result

    def disable_agent(self):
        system_id = self.get_system_id()
        if system_id is None:
            print("Warning: Cannot disable user-agent because system_id is None.")
            return
        self.exec_command("{} {} --system_id {}"
                          .format(XRAY_MANAGER_UTILITY, "disable-user-agent", system_id))

    def enable_agent(self):
        system_id = self.get_system_id()
        if system_id is None:
            print("Warning: Cannot enable user-agent because system_id is None.")
            return
        self.exec_command("{} {} --system_id {}"
                          .format(XRAY_MANAGER_UTILITY, "enable-user-agent", system_id))

    def is_agent_enabled_in_config(self):
        return UIConfig().get_param('hideXrayApp', 'uiSettings') is False

    def sync_ui_config(self):
        self.exec_command(DYNAMICUI_SYNC_CONFIG_COMMAND)

    def configure_plugin(self):
        self.sync_ui_config()
        if self.is_agent_enabled_in_config():
            self.enable_agent()
        else:
            self.disable_agent()

    def get_system_id(self):
        try:
            system_id = get_rhn_systemid_value('system_id').split('-')[1]
            return system_id
        except Exception as e:
            print("Warning: Cannot get system id. It looks like your system is not registered.")
            return


    def reset_ui_config(self):
        if not is_cl_solo_edition(skip_jwt_check=True):
            self.exec_command(CLOUDLINUX_CONFIG_COMMAND)
        self.exec_command(DYNAMICUI_SYNC_CONFIG_COMMAND)

    def remove_file(self, filename):
        try:
            os.remove(filename)
        except OSError:
            pass


class CpanelPluginInstaller(Base):
    def __init__(self):
        self.ROOT_CPANEL_DIR = "/usr/local/cpanel/whostmgr/docroot/"
        self.CPANEL_PAPER_LATERN_HOME = "/usr/local/cpanel/base/frontend/paper_lantern/"
        self.config_js_src = SOURCE_PATH + "plugins/cpanel/xray_config.js"
        self.config_js_dst = self.ROOT_CPANEL_DIR + "3rdparty/cloudlinux/assets/js/xray_config.js"
        self.bundle_src = SOURCE_PATH + "dist/xray.bundle.min.js"
        self.bundle_dst = self.ROOT_CPANEL_DIR + "3rdparty/cloudlinux/assets/static/xray.bundle.min.js"
        self.template_src = SOURCE_PATH + "plugins/cpanel/xray.live.pl"
        self.template_dst = self.CPANEL_PAPER_LATERN_HOME + "lveversion/xray.live.pl"
        self.plugin_tar = SOURCE_PATH + "dist/cpanel-xray-plugin.tar.bz2"
        self.plugin_installer = "/usr/local/cpanel/scripts/install_plugin"
        self.plugin_uninstaller = "/usr/local/cpanel/scripts/uninstall_plugin"
        self.xray_feature_file = "/usr/local/cpanel/whostmgr/addonfeatures/lvexray"
        self.install_config = SOURCE_PATH + "plugins/cpanel/plugin/install.json"

    def cpanel_fix_feature_manager(self):
        if os.path.exists(self.xray_feature_file) and os.path.exists(self.install_config):
            with open(self.install_config) as install_config:
                feature_config = json.load(install_config)
                feature = feature_config[0]  # We have only one item
                try:
                    feature_name_fixed = re.search("\$LANG{'(.*)'}", feature['name']).group(1)
                except AttributeError:
                    feature_name_fixed = ''
                feature_file = open(self.xray_feature_file, 'w')
                feature_file.write(feature['feature'] + ':' + feature_name_fixed)
                feature_file.close()

    def install_plugin(self):
        shutil.copy(self.config_js_src, self.config_js_dst)
        shutil.copy(self.bundle_src, self.bundle_dst)
        shutil.copy(self.template_src, self.template_dst)
        if os.path.exists(self.plugin_tar) and os.path.exists(self.plugin_installer):
            self.exec_command("{} {}  --theme paper_lantern".format(self.plugin_installer, self.plugin_tar))
        self.cpanel_fix_feature_manager()

    def uninstall_plugin(self):
        self.remove_file(self.config_js_dst)
        self.remove_file(self.bundle_dst)
        self.remove_file(self.template_dst)
        if os.path.exists(self.plugin_tar) and os.path.exists(self.plugin_uninstaller):
            self.exec_command("{} {}  --theme paper_lantern".format(self.plugin_uninstaller, self.plugin_tar))


class PleskPluginInstaller(Base):
    def __init__(self):
        self.ROOT_PLESK_DIR = "/usr/local/psa/admin/"
        self.controller_src = SOURCE_PATH + "plugins/plesk/XrayController.php"
        self.controller_dst = self.ROOT_PLESK_DIR + "plib/modules/plesk-lvemanager/controllers/XrayController.php"
        self.send_request_controller_src = SOURCE_PATH + "plugins/plesk/XraySendRequestController.php"
        self.send_request_controller_dst = self.ROOT_PLESK_DIR + "plib/modules/plesk-lvemanager/controllers/XraySendRequestController.php"
        self.bundle_src = SOURCE_PATH + "dist/xray.bundle.min.js"
        self.bundle_dst = self.ROOT_PLESK_DIR + "htdocs/modules/plesk-lvemanager/static/xray.bundle.min.js"
        self.icon_src = SOURCE_PATH + "plugins/plesk/xray.svg"
        self.icon_dst = self.ROOT_PLESK_DIR + "htdocs/modules/plesk-lvemanager/images/xray.svg"
        self.views_dir = self.ROOT_PLESK_DIR + "plib/modules/plesk-lvemanager/views/scripts/xray/"
        self.template_src = SOURCE_PATH + "plugins/plesk/index.phtml"
        self.template_dst = self.ROOT_PLESK_DIR + "plib/modules/plesk-lvemanager/views/scripts/xray/index.phtml"

    def install_plugin(self):
        shutil.copy(self.controller_src, self.controller_dst)
        shutil.copy(self.send_request_controller_src, self.send_request_controller_dst)
        shutil.copy(self.bundle_src, self.bundle_dst)
        shutil.copy(self.icon_src, self.icon_dst)
        if not os.path.isdir(self.views_dir):
            os.mkdir(self.views_dir)
        shutil.copy(self.template_src, self.template_dst)

    def uninstall_plugin(self):
        self.remove_file(self.controller_dst)
        self.remove_file(self.send_request_controller_dst)
        self.remove_file(self.bundle_dst)
        self.remove_file(self.icon_dst)
        shutil.rmtree(self.views_dir)


class DirectAdminPluginInstaller(Base):
    def __init__(self):
        self.ROOT_DA_DIR = "/usr/local/directadmin/plugins/"
        self.plugin_dir_src = SOURCE_PATH + 'plugins/directadmin/xray'
        self.plugin_dir_dst = self.ROOT_DA_DIR + 'xray'
        self.assets_src = self.ROOT_DA_DIR + 'lvemanager_spa/images/assets'
        self.assets_dst = self.ROOT_DA_DIR + 'xray/images/assets'
        self.bundle_src = SOURCE_PATH + 'dist/xray.bundle.min.js'
        self.bundle_dst = self.ROOT_DA_DIR + 'xray/images/assets/static/xray.bundle.min.js'
        self.plugin_config = self.ROOT_DA_DIR + "xray/plugin.conf"

    def install_plugin(self):
        if os.path.exists(self.plugin_dir_dst):
            shutil.rmtree(self.plugin_dir_dst)
        shutil.copytree(self.plugin_dir_src, self.plugin_dir_dst)
        shutil.copytree(self.assets_src, self.assets_dst)
        shutil.copy(self.bundle_src, self.bundle_dst)
        self.exec_command("chown -R diradmin:diradmin " + self.plugin_dir_dst)
        self.exec_command("chmod -R 755 " + self.plugin_dir_dst)
        self.exec_command("chmod -R 644 " + self.plugin_config)

    def uninstall_plugin(self):
        shutil.rmtree(self.plugin_dir_dst)


class PanelIntegrationPluginInstaller(Base):
    def __init__(self):
        self.bundle_src = SOURCE_PATH + "dist/xray.bundle.min.js"
        self.bundle_dst = LVEMANAGER_SOURCE_PATH + "commons/spa-resources/static/xray.bundle.min.js"
        self.bundle_base_dst = "/assets/static/xray.bundle.min.js"

    def get_panel_base_path(self):
        try:
            parser = configparser.ConfigParser(interpolation=None, strict=False)
            parser.read(PANEL_INTEGRATION_CONFIG)
            base_path = parser.get("lvemanager_config", "base_path")
            return base_path
        except Exception as e:
            print('Cannot copy files for no panel version. {}'.format(e))
            sys.exit(1)

    def install_plugin(self):
        base_path = self.get_panel_base_path().rstrip('/')
        shutil.copy(self.bundle_src, self.bundle_dst)
        shutil.copy(self.bundle_src, base_path + self.bundle_base_dst)

    def uninstall_plugin(self):
        base_path = self.get_panel_base_path().rstrip('/')
        self.remove_file(self.bundle_dst)
        self.remove_file(base_path + self.bundle_base_dst)


class Main:
    def __init__(self):
        pass

    def make_parser(self):
        parser = argparse.ArgumentParser(description="Script to install|uninstall Xray App for user")
        parser.add_argument("--install", "-i", action="store_true",
                            help="Install Xray App")
        parser.add_argument("--uninstall", "-u", action="store_true",
                            help="Uninstall Xray App")
        return parser

    def run(self):
        parser = self.make_parser()
        args = parser.parse_args()
        installer_class = None

        if detect.is_cpanel():
            # Cpanel
            installer = CpanelPluginInstaller()
        elif detect.is_da():
            # DirectAdmin
            installer = DirectAdminPluginInstaller()
        elif detect.is_plesk():
            # Plesk
            installer = PleskPluginInstaller()
        elif os.path.isfile(PANEL_INTEGRATION_CONFIG):
             # Custom panel with integration.ini
             installer = PanelIntegrationPluginInstaller()
        else:
            print("X-Ray plugin cannot be installed on your environment")
            sys.exit(0)

        if args.install:
            installer.install_plugin()
            installer.configure_plugin()
        elif args.uninstall:
            installer.uninstall_plugin()
            installer.reset_ui_config()
        else:
            parser.print_help()


if __name__ == "__main__":
    main = Main()
    main.run()
